(*---------------------------------------------------------------------------
   Copyright (c) 2014 Daniel C. Bünzli. All rights reserved.
   Distributed under the BSD3 license, see license at the end of the file.
   uuseg release 0.9.0
  ---------------------------------------------------------------------------*)

let dummy_u = `Uchar 0x0000              (* to initalize arrays, never read. *)

type t =
  { mutable buf : [`Uchar of int] array;
    mutable first : int;
    mutable last : int; }

let create n = { buf = Array.create n dummy_u; first = 0; last = -1; }
let copy b = { b with buf = Array.copy b.buf; }
let empty b = b.last = -1
let len b = (b.last - b.first) + 1
let grow b =
  let len = Array.length b.buf in
  let newbuf = Array.create (2 * len) dummy_u in
  Array.blit b.buf 0 newbuf 0 len; b.buf <- newbuf

let add b add =
  let last = b.last + 1 in
  if last = Array.length b.buf then grow b;
  b.buf.(last) <- add; b.last <- last

let flush b =
  let `Uchar _ as add = b.buf.(b.first) in
  b.first <- b.first + 1;
  if b.first > b.last then (b.first <- 0; b.last <- -1);
  add

(*---------------------------------------------------------------------------
   Copyright (c) 2014 Daniel C. Bünzli.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

   1. Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above
      copyright notice, this list of conditions and the following
      disclaimer in the documentation and/or other materials provided
      with the distribution.

   3. Neither the name of Daniel C. Bünzli nor the names of
      contributors may be used to endorse or promote products derived
      from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  ---------------------------------------------------------------------------*)
